@@ -246,6 +246,30 @@ UserData *UserFromInputMTP(
 	});
 }
 
+Ui::ColorCollectible ParseColorCollectible(
+		const MTPDpeerColorCollectible &data) {
+	return {
+		.collectibleId = data.vcollectible_id().v,
+		.giftEmojiId = data.vgift_emoji_id().v,
+		.backgroundEmojiId = data.vbackground_emoji_id().v,
+		.accentColor = Ui::ColorFromSerialized(data.vaccent_color()),
+		.strip = ranges::views::all(
+			data.vcolors().v
+		) | ranges::views::transform(
+			&Ui::ColorFromSerialized
+		) | ranges::to_vector,
+		.darkAccentColor = Ui::MaybeColorFromSerialized(
+			data.vdark_accent_color()).value_or(QColor(0, 0, 0, 0)),
+		.darkStrip = (data.vdark_colors()
+			? ranges::views::all(
+				data.vdark_colors()->v
+			) | ranges::views::transform(
+				&Ui::ColorFromSerialized
+			) | ranges::to_vector
+			: std::vector<QColor>()),
+	};
+}
+
 } // namespace Data
 
 PeerClickHandler::PeerClickHandler(not_null<PeerData*> peer)
@@ -693,7 +717,9 @@ bool PeerData::canCreateTodoLists() const {
 }
 
 bool PeerData::canCreateTopics() const {
-	if (const auto channel = asChannel()) {
+	if (const auto bot = asBot()) {
+		return bot->isForum();
+	} else if (const auto channel = asChannel()) {
 		return channel->isForum()
 			&& !channel->amRestricted(ChatRestriction::CreateTopics);
 	}
@@ -701,7 +727,9 @@ bool PeerData::canCreateTopics() const {
 }
 
 bool PeerData::canManageTopics() const {
-	if (const auto channel = asChannel()) {
+	if (const auto bot = asBot()) {
+		return bot->isForum();
+	} else if (const auto channel = asChannel()) {
 		return channel->isForum()
 			&& (channel->amCreator()
 				|| (channel->adminRights() & ChatAdminRight::ManageTopics));
@@ -924,6 +952,13 @@ void PeerData::setBarSettings(PeerBarSettings which) {
 				history->refreshHiddenLinksItems();
 			});
 		}
+		if (const auto from = migrateFrom()) {
+			if (const auto history = owner().historyLoaded(from)) {
+				crl::on_main(&history->session(), [=] {
+					history->refreshHiddenLinksItems();
+				});
+			}
+		}
 	}
 }
 
@@ -952,6 +987,9 @@ bool PeerData::hideLinks() const {
 	//if (!isUser()) {
 	//	return false;
 	//}
+	if (const auto to = migrateTo()) {
+		return to->hideLinks();
+	}
 	const auto settings = barSettings();
 	return !settings || (*settings & PeerBarSetting::ReportSpam);
 }
@@ -994,13 +1032,6 @@ TimeId PeerData::photoChangeDate() const {
 	return _barDetails ? _barDetails->photoChangeDate : 0;
 }
 
-bool PeerData::changeColorIndex(
-		const tl::conditional<MTPint> &cloudColorIndex) {
-	return cloudColorIndex
-		? changeColorIndex(cloudColorIndex->v)
-		: clearColorIndex();
-}
-
 bool PeerData::changeBackgroundEmojiId(
 		const tl::conditional<MTPlong> &cloudBackgroundEmoji) {
 	return changeBackgroundEmojiId(cloudBackgroundEmoji
@@ -1008,15 +1039,40 @@ bool PeerData::changeBackgroundEmojiId(
 		: DocumentId());
 }
 
+bool PeerData::changeColorCollectible(
+		const tl::conditional<MTPPeerColor> &cloudColor) {
+	if (!cloudColor) {
+		return clearColorCollectible();
+	}
+	return cloudColor->match([&](const MTPDpeerColorCollectible &data) {
+		return changeColorCollectible(Data::ParseColorCollectible(data));
+	}, [&](const MTPDpeerColor &) {
+		return clearColorCollectible();
+	}, [&](const MTPDinputPeerColorCollectible &) {
+		return clearColorCollectible();
+	});
+}
+
 bool PeerData::changeColor(
 		const tl::conditional<MTPPeerColor> &cloudColor) {
 	const auto changed1 = cloudColor
-		? changeColorIndex(cloudColor->data().vcolor())
+		? changeColorIndex(Data::ColorIndexFromColor(cloudColor))
 		: clearColorIndex();
-	const auto changed2 = changeBackgroundEmojiId(cloudColor
-		? cloudColor->data().vbackground_emoji_id().value_or_empty()
-		: DocumentId());
-	return changed1 || changed2;
+	const auto changed2 = changeBackgroundEmojiId(
+		Data::BackgroundEmojiIdFromColor(cloudColor));
+	const auto changed3 = changeColorCollectible(cloudColor);
+	return changed1 || changed2 || changed3;
+}
+
+bool PeerData::changeColorProfile(
+		const tl::conditional<MTPPeerColor> &cloudColor) {
+	const auto changed1 = cloudColor
+		? changeColorProfileIndex(Data::ColorIndexFromColor(cloudColor))
+		: clearColorProfileIndex();
+	const auto changed2 = changeProfileBackgroundEmojiId(
+		Data::BackgroundEmojiIdFromColor(cloudColor));
+	const auto changed3 = changeColorProfileCollectible(cloudColor);
+	return changed1 || changed2 || changed3;
 }
 
 void PeerData::fillNames() {
@@ -1325,6 +1381,25 @@ bool PeerData::isUsernameEditable(QString username) const {
 	return false;
 }
 
+bool PeerData::changeColorCollectible(Ui::ColorCollectible data) {
+	if (!_colorCollectible || (*_colorCollectible != data)) {
+		// We don't reuse allocated object because in ChatStyle we
+		// cache colors using std::weak_ptr as a key.
+		_colorCollectible = std::make_shared<Ui::ColorCollectible>(
+			std::move(data));
+		return true;
+	}
+	return false;
+}
+
+bool PeerData::clearColorCollectible() {
+	if (!_colorCollectible) {
+		return false;
+	}
+	_colorCollectible = nullptr;
+	return true;
+}
+
 bool PeerData::changeColorIndex(uint8 index) {
 	index %= Ui::kColorIndexCount;
 	if (_colorIndexCloud && _colorIndex == index) {
@@ -1356,6 +1431,66 @@ bool PeerData::changeBackgroundEmojiId(DocumentId id) {
 	return true;
 }
 
+bool PeerData::changeColorProfileCollectible(Ui::ColorCollectible data) {
+	if (!_colorProfileCollectible || (*_colorProfileCollectible != data)) {
+		_colorProfileCollectible = std::make_shared<Ui::ColorCollectible>(
+			std::move(data));
+		return true;
+	}
+	return false;
+}
+
+bool PeerData::changeColorProfileCollectible(
+		const tl::conditional<MTPPeerColor> &cloudColor) {
+	if (!cloudColor) {
+		return clearColorProfileCollectible();
+	}
+	return cloudColor->match([&](const MTPDpeerColorCollectible &data) {
+		return changeColorProfileCollectible(Data::ParseColorCollectible(data));
+	}, [&](const MTPDpeerColor &) {
+		return clearColorProfileCollectible();
+	}, [&](const MTPDinputPeerColorCollectible &) {
+		return clearColorProfileCollectible();
+	});
+}
+
+bool PeerData::clearColorProfileCollectible() {
+	if (!_colorProfileCollectible) {
+		return false;
+	}
+	_colorProfileCollectible = nullptr;
+	return true;
+}
+
+bool PeerData::changeColorProfileIndex(uint8 index) {
+	index %= Ui::kColorIndexCount;
+	if (_colorProfileIndex == index) {
+		return false;
+	}
+	_colorProfileIndex = index;
+	return true;
+}
+
+bool PeerData::clearColorProfileIndex() {
+	if (!_colorProfileIndex.has_value()) {
+		return false;
+	}
+	_colorProfileIndex = std::nullopt;
+	return true;
+}
+
+DocumentId PeerData::profileBackgroundEmojiId() const {
+	return _profileBackgroundEmojiId;
+}
+
+bool PeerData::changeProfileBackgroundEmojiId(DocumentId id) {
+	if (_profileBackgroundEmojiId == id) {
+		return false;
+	}
+	_profileBackgroundEmojiId = id;
+	return true;
+}
+
 void PeerData::setEmojiStatus(const MTPEmojiStatus &status) {
 	const auto parsed = owner().emojiStatuses().parse(status);
 	setEmojiStatus(parsed.id, parsed.until);
@@ -1436,7 +1571,9 @@ bool PeerData::isBroadcast() const {
 }
 
 bool PeerData::isForum() const {
-	if (const auto channel = asChannel()) {
+	if (const auto bot = asBot()) {
+		return bot->isForum();
+	} else if (const auto channel = asChannel()) {
 		return channel->isForum();
 	}
 	return false;
@@ -1517,7 +1654,9 @@ Ui::BotVerifyDetails *PeerData::botVerifyDetails() const {
 }
 
 Data::Forum *PeerData::forum() const {
-	if (const auto channel = asChannel()) {
+	if (const auto bot = asBot()) {
+		return bot->forum();
+	} else if (const auto channel = asChannel()) {
 		return channel->forum();
 	}
 	return nullptr;
@@ -1549,6 +1688,28 @@ Data::SavedSublist *PeerData::monoforumSublistFor(
 	return nullptr;
 }
 
+bool PeerData::useSubsectionTabs() const {
+	if (const auto bot = asBot()) {
+		return bot->isForum();
+	} else if (const auto channel = asChannel()) {
+		return channel->useSubsectionTabs();
+	}
+	return false;
+}
+
+bool PeerData::viewForumAsMessages() const {
+	if (const auto channel = asChannel()) {
+		return channel->viewForumAsMessages();
+	}
+	return false;
+}
+
+void PeerData::processTopics(const MTPVector<MTPForumTopic> &topics) {
+	if (const auto forum = this->forum()) {
+		forum->applyReceivedTopics(topics);
+	}
+}
+
 bool PeerData::allowsForwarding() const {
 	if (isUser()) {
 		return true;
@@ -1691,7 +1852,9 @@ int PeerData::slowmodeSecondsLeft() const {
 }
 
 bool PeerData::canManageGroupCall() const {
-	if (const auto chat = asChat()) {
+	if (const auto user = asUser()) {
+		return user->isSelf();
+	} else if (const auto chat = asChat()) {
 		return chat->amCreator()
 			|| (chat->adminRights() & ChatAdminRight::ManageCall);
 	} else if (const auto group = asChannel()) {
@@ -1701,7 +1864,7 @@ bool PeerData::canManageGroupCall() const {
 		return group->amCreator()
 			|| (group->adminRights() & ChatAdminRight::ManageCall);
 	}
-	return false;
+	Unexpected("Peer type in PeerData::canManageGroupCall.");
 }
 
 bool PeerData::amMonoforumAdmin() const {
@@ -1823,6 +1986,15 @@ bool PeerData::hasUnreadStories() const {
 	return false;
 }
 
+bool PeerData::hasActiveVideoStream() const {
+	if (const auto user = asUser()) {
+		return user->hasActiveVideoStream();
+	} else if (const auto channel = asChannel()) {
+		return channel->hasActiveVideoStream();
+	}
+	return false;
+}
+
 void PeerData::setStoriesState(StoriesState state) {
 	if (const auto user = asUser()) {
 		return user->setStoriesState(state);
@@ -1987,4 +2159,30 @@ FullMsgId ResolveMinPinnedId(
 	}
 }
 
+uint64 BackgroundEmojiIdFromColor(const MTPPeerColor *color) {
+	if (!color) {
+		return 0;
+	}
+	return color->match([](const MTPDpeerColor &data) -> uint64 {
+		return data.vbackground_emoji_id().value_or_empty();
+	}, [](const MTPDpeerColorCollectible &data) -> uint64 {
+		return data.vbackground_emoji_id().v;
+	}, [](const MTPDinputPeerColorCollectible &data) -> uint64 {
+		return 0;
+	});
+}
+
+uint8 ColorIndexFromColor(const MTPPeerColor *color) {
+	if (!color) {
+		return 0;
+	}
+	return color->match([](const MTPDpeerColor &data) -> uint8 {
+		return data.vcolor().value_or_empty();
+	}, [](const MTPDpeerColorCollectible &data) -> uint8 {
+		return 0;
+	}, [](const MTPDinputPeerColorCollectible &) -> uint8 {
+		return 0;
+	});
+}
+
 } // namespace Data
