@@ -11,19 +11,27 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "apiwrap.h"
 #include "base/unixtime.h"
 #include "base/qt/qt_key_modifiers.h"
+#include "boxes/choose_filter_box.h"
 #include "boxes/peer_list_box.h"
 #include "core/application.h"
+#include "core/ui_integration.h"
 #include "data/components/recent_peers.h"
 #include "data/components/top_peers.h"
+#include "data/stickers/data_custom_emoji.h"
 #include "data/data_changes.h"
 #include "data/data_channel.h"
 #include "data/data_chat.h"
+#include "data/data_chat_filters.h"
 #include "data/data_download_manager.h"
 #include "data/data_folder.h"
 #include "data/data_peer_values.h"
 #include "data/data_session.h"
 #include "data/data_user.h"
 #include "dialogs/ui/chat_search_empty.h"
+#include "dialogs/ui/chat_search_in.h"
+#include "dialogs/ui/posts_search_intro.h"
+#include "dialogs/dialogs_inner_widget.h"
+#include "dialogs/dialogs_search_posts.h"
 #include "history/history.h"
 #include "info/downloads/info_downloads_widget.h"
 #include "info/media/info_media_widget.h"
@@ -34,10 +42,15 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "lang/lang_keys.h"
 #include "main/main_session.h"
 #include "settings/settings_common.h"
+#include "settings/settings_credits_graphics.h"
+#include "settings/settings_premium.h"
 #include "storage/storage_shared_media.h"
 #include "ui/boxes/confirm_box.h"
 #include "ui/controls/swipe_handler.h"
 #include "ui/effects/ripple_animation.h"
+#include "ui/toast/toast.h"
+#include "ui/text/custom_emoji_helper.h"
+#include "ui/text/custom_emoji_text_badge.h"
 #include "ui/text/text_utilities.h"
 #include "ui/widgets/menu/menu_add_action_callback_factory.h"
 #include "ui/widgets/buttons.h"
@@ -57,10 +70,12 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "window/window_separate_id.h"
 #include "window/window_session_controller.h"
 #include "window/window_peer_menu.h"
+#include "styles/style_boxes.h"
 #include "styles/style_chat.h"
 #include "styles/style_dialogs.h"
 #include "styles/style_layers.h"
 #include "styles/style_menu_icons.h"
+#include "styles/style_settings.h"
 #include "styles/style_window.h"
 
 namespace Dialogs {
@@ -130,6 +145,7 @@ struct EntryMenuDescriptor {
 	QString removeAllText;
 	QString removeAllConfirm;
 	Fn<void()> removeAll;
+	Fn<void()> closeCallback;
 };
 
 [[nodiscard]] Fn<void()> RemoveAllConfirm(
@@ -155,6 +171,9 @@ void FillEntryMenu(
 	add(tr::lng_context_new_window(tr::now), [=] {
 		Ui::PreventDelayedActivation();
 		controller->showInNewWindow(peer);
+		if (descriptor.closeCallback) {
+			descriptor.closeCallback();
+		}
 	}, &st::menuIconNewWindow);
 	Window::AddSeparatorAndShiftUp(add);
 
@@ -167,6 +186,20 @@ void FillEntryMenu(
 		controller->showPeerHistory(peer);
 	}, channel ? &st::menuIconChannel : &st::menuIconChatBubble);
 
+	const auto history = peer->owner().historyLoaded(peer);
+	if (history
+		&& history->owner().chatsFilters().has()
+		&& history->inChatList()) {
+		add(Ui::Menu::MenuCallback::Args{
+			.text = tr::lng_filters_menu_add(tr::now),
+			.handler = nullptr,
+			.icon = &st::menuIconAddToFolder,
+			.fillSubmenu = [&](not_null<Ui::PopupMenu*> menu) {
+				FillChooseFilterMenu(controller, menu, history);
+			},
+			.submenuSt = &st::foldersMenu,
+		});
+	}
 	const auto viewProfileText = group
 		? tr::lng_context_view_group(tr::now)
 		: channel
@@ -409,6 +442,10 @@ public:
 		return _chosen.events();
 	}
 
+	void setCloseCallback(Fn<void()> callback) {
+		_closeCallback = std::move(callback);
+	}
+
 	Main::Session &session() const override {
 		return _window->session();
 	}
@@ -432,6 +469,8 @@ protected:
 	void setupPlainDivider(rpl::producer<QString> title);
 	void setupExpandDivider(rpl::producer<QString> title);
 
+	Fn<void()> _closeCallback;
+
 private:
 	const not_null<Window::SessionController*> _window;
 
@@ -451,7 +490,8 @@ public:
 	RecentsController(
 		not_null<Window::SessionController*> window,
 		RecentPeersList list,
-		RightActionCallback rightActionCallback);
+		RightActionCallback rightActionCallback,
+		Fn<void()> closeCallback);
 
 	void prepare() override;
 	base::unique_qptr<Ui::PopupMenu> rowContextMenu(
@@ -675,6 +715,9 @@ void Suggestions::ObjectListController::rowClicked(
 void Suggestions::ObjectListController::rowMiddleClicked(
 		not_null<PeerListRow*> row) {
 	window()->showInNewWindow(row->peer());
+	if (_closeCallback) {
+		_closeCallback();
+	}
 }
 
 void Suggestions::ObjectListController::setupPlainDivider(
@@ -771,10 +814,12 @@ void Suggestions::ObjectListController::setupExpandDivider(
 RecentsController::RecentsController(
 	not_null<Window::SessionController*> window,
 	RecentPeersList list,
-	RightActionCallback rightActionCallback)
+	RightActionCallback rightActionCallback,
+	Fn<void()> closeCallback)
 : ObjectListController(window)
 , _recent(std::move(list))
 , _rightActionCallback(std::move(rightActionCallback)) {
+	_closeCallback = std::move(closeCallback);
 }
 
 void RecentsController::prepare() {
@@ -832,6 +877,11 @@ base::unique_qptr<Ui::PopupMenu> RecentsController::rowContextMenu(
 		.removeAllText = tr::lng_recent_clear_all(tr::now),
 		.removeAllConfirm = tr::lng_recent_clear_sure(tr::now),
 		.removeAll = removeAllCallback(),
+		.closeCallback = crl::guard(this, [=] {
+			if (_closeCallback) {
+				_closeCallback();
+			}
+		}),
 	});
 	return result;
 }
@@ -1181,6 +1231,11 @@ base::unique_qptr<Ui::PopupMenu> RecentAppsController::rowContextMenu(
 		.peer = peer,
 		.removeOneText = tr::lng_recent_remove(tr::now),
 		.removeOne = removeOne,
+		.closeCallback = crl::guard(this, [=] {
+			if (_closeCallback) {
+				_closeCallback();
+			}
+		}),
 	});
 	return result;
 }
@@ -1331,6 +1386,8 @@ Suggestions::Suggestions(
 , _appsScroll(std::make_unique<Ui::ElasticScroll>(this))
 , _appsContent(
 	_appsScroll->setOwnedWidget(object_ptr<Ui::VerticalLayout>(this)))
+, _postsScroll(std::make_unique<Ui::ElasticScroll>(this))
+, _postsWrap(_postsScroll->setOwnedWidget(object_ptr<Ui::RpWidget>(this)))
 , _recentApps(setupRecentApps())
 , _popularApps(setupPopularApps())
 , _searchQueryTimer([=] { applySearchQuery(); }) {
@@ -1397,6 +1454,7 @@ void Suggestions::setupTabs() {
 		{ Key{ Tab::Chats }, tr::lng_recent_chats(tr::now) },
 		{ Key{ Tab::Channels }, tr::lng_recent_channels(tr::now) },
 		{ Key{ Tab::Apps }, tr::lng_recent_apps(tr::now) },
+		{ Key{ Tab::Posts }, tr::lng_recent_posts(tr::now) },
 		{ Key{ Tab::Media, MediaType::Photo }, tr::lng_all_photos(tr::now) },
 		{ Key{ Tab::Media, MediaType::Video }, tr::lng_all_videos(tr::now) },
 		{ Key{ Tab::Downloads }, tr::lng_all_downloads(tr::now) },
@@ -1411,11 +1469,12 @@ void Suggestions::setupTabs() {
 			tr::lng_all_voice(tr::now),
 		},
 	};
-	auto sections = std::vector<QString>();
+
+	auto sections = std::vector<TextWithEntities>();
 	for (const auto key : _tabKeys) {
 		const auto i = labels.find(key);
 		Assert(i != end(labels));
-		sections.push_back(i->second);
+		sections.push_back({ i->second });
 	}
 	_tabs->setSections(sections);
 	_tabs->sectionActivated(
@@ -1478,7 +1537,10 @@ void Suggestions::setupChats() {
 				Ui::Text::FixAmpersandInAction),
 			.removeAllConfirm = tr::lng_recent_hide_sure(tr::now),
 			.removeAll = removeAll,
-		});
+			.closeCallback = crl::guard(
+				this,
+				[=] { _closeRequests.fire({}); }),
+			});
 	}, _topPeers->lifetime());
 
 	_topPeers->scrollToRequests(
@@ -1496,8 +1558,8 @@ void Suggestions::setupChats() {
 }
 
 void Suggestions::handlePressForChatPreview(
-		PeerId id,
-		Fn<void(bool)> callback) {
+	PeerId id,
+	Fn<void(bool)> callback) {
 	callback = crl::guard(this, callback);
 	const auto row = RowDescriptor(
 		_controller->session().data().history(id),
@@ -1812,7 +1874,11 @@ bool Suggestions::consumeSearchQuery(const QString &query) {
 	const auto key = _key.current();
 	const auto tab = key.tab;
 	const auto type = (key.tab == Tab::Media) ? key.mediaType : Type::kCount;
-	if (tab != Tab::Downloads
+	if (tab == Tab::Posts) {
+		const auto changed = (_searchQuery != query);
+		setPostsSearchQuery(query);
+		return changed || !query.isEmpty();
+	} else if (tab != Tab::Downloads
 		&& type != Type::File
 		&& type != Type::Link
 		&& type != Type::MusicFile) {
@@ -1831,6 +1897,178 @@ bool Suggestions::consumeSearchQuery(const QString &query) {
 	return true;
 }
 
+void Suggestions::setupPostsSearch() {
+	_postsSearch = std::make_unique<PostsSearch>(&_controller->session());
+
+	_postsSearch->stateUpdates(
+	) | rpl::start_with_next([=](const PostsSearchState &state) {
+		if (state.intro) {
+			if (!_postsSearchIntro) {
+				setupPostsIntro(*state.intro);
+			} else {
+				_postsSearchIntro->update(*state.intro);
+			}
+			return;
+		} else if (!_postsContent) {
+			setupPostsResults();
+		}
+
+		_postsContent->applySearchState(SearchState{
+			.tab = ChatSearchTab::PublicPosts,
+			.query = _searchQuery,
+		});
+		if (state.loading) {
+			_postsContent->searchRequested(true);
+		} else {
+			_postsContent->searchReceived(
+				state.page,
+				nullptr,
+				{ .posts = true, .start = true },
+				state.totalCount);
+			_postsScroll->scrollToY(0);
+			updatePostsSearchVisibleRange();
+		}
+	}, _postsWrap->lifetime());
+
+	_postsSearch->pagesUpdates(
+	) | rpl::start_with_next([=](const PostsSearchState &state) {
+		Expects(!state.intro && !state.loading);
+
+		if (!_postsContent) {
+			return;
+		}
+		_postsContent->searchReceived(
+			state.page,
+			nullptr,
+			{ .posts = true },
+			state.totalCount);
+		updatePostsSearchVisibleRange();
+	}, _postsWrap->lifetime());
+}
+
+void Suggestions::setPostsSearchQuery(const QString &query) {
+	if (!_postsSearch) {
+		setupPostsSearch();
+	}
+	if (!query.isEmpty()) {
+		_persist = true;
+	}
+	_searchQuery = query;
+	_searchQueryTimer.cancel();
+	_postsSearch->setQuery(query);
+}
+
+void Suggestions::setupPostsResults() {
+	Expects(!_postsContent);
+
+	delete base::take(_postsSearchIntro);
+	_postsContent = Ui::CreateChild<InnerWidget>(
+		_postsWrap.get(),
+		_controller,
+		rpl::single(InnerWidget::ChildListShown()));
+
+	_postsContent->applySearchState(SearchState{
+		.tab = ChatSearchTab::PublicPosts,
+		.query = _searchQuery,
+	});
+	_postsContent->searchRequested(true);
+
+	_postsContent->chosenRow(
+	) | rpl::start_with_next([=](const ChosenRow &row) {
+		const auto history = row.key.history();
+		if (!history) {
+			return;
+		}
+		_persist = true;
+		const auto showAtMsgId = row.message.fullId.msg;
+		auto params = Window::SectionShow(
+			Window::SectionShow::Way::ClearStack);
+		params.highlight = Window::SearchHighlightId(_searchQuery);
+		if (row.newWindow) {
+			_controller->showInNewWindow(history->peer, showAtMsgId);
+			_closeRequests.fire({});
+		} else {
+			_controller->showThread(history, showAtMsgId, params);
+		}
+	}, _postsContent->lifetime());
+
+	_postsContent->heightValue() | rpl::start_with_next([=](int height) {
+		_postsWrap->resize(_postsWrap->width(), height);
+	}, _postsContent->lifetime());
+
+	rpl::combine(
+		rpl::single(rpl::empty) | rpl::then(_postsScroll->scrolls()),
+		_postsScroll->heightValue()
+	) | rpl::start_with_next([=] {
+		updatePostsSearchVisibleRange();
+	}, _postsContent->lifetime());
+
+	_postsContent->setLoadMoreCallback([=] {
+		_postsSearch->requestMore();
+	});
+
+	_postsContent->setNarrowRatio(0.);
+	_postsContent->show();
+	updateControlsGeometry();
+}
+
+void Suggestions::updatePostsSearchVisibleRange() {
+	Expects(_postsContent != nullptr);
+
+	const auto top = _postsScroll->scrollTop();
+	const auto height = _postsScroll->height();
+	_postsContent->setVisibleTopBottom(top, top + height);
+}
+
+void Suggestions::setupPostsIntro(const PostsSearchIntroState &intro) {
+	Expects(!_postsSearchIntro);
+
+	delete base::take(_postsContent);
+	_postsSearchIntro = Ui::CreateChild<PostsSearchIntro>(_postsWrap, intro);
+
+	_postsSearchIntro->searchWithStars(
+	) | rpl::start_with_next([=](int stars) {
+		if (!_controller->session().premium()) {
+			Settings::ShowPremium(
+				_controller,
+				u"posts_search"_q);
+		} else if (!stars) {
+			_postsSearch->setAllowedStars(0);
+		} else {
+			using namespace Settings;
+			const auto done = [=](Settings::SmallBalanceResult result) {
+				if (result == Settings::SmallBalanceResult::Success
+					|| result == Settings::SmallBalanceResult::Already) {
+					const auto spent = _postsSearch->setAllowedStars(stars);
+					if (spent > 0) {
+						_controller->showToast({
+							.text = tr::lng_posts_paid_spent(
+								tr::now,
+								lt_count,
+								spent,
+								Ui::Text::RichLangValue),
+							.attach = RectPart::Top,
+							.duration = Ui::Toast::kDefaultDuration * 2,
+						});
+					}
+				}
+			};
+			MaybeRequestBalanceIncrease(
+				_controller->uiShow(),
+				stars,
+				SmallBalanceForSearch{},
+				done);
+		}
+	}, _postsSearchIntro->lifetime());
+
+	_postsScroll->heightValue() | rpl::start_with_next([=](int height) {
+		_postsWrap->resize(_postsWrap->width(), height);
+	}, _postsSearchIntro->lifetime());
+
+	_postsSearchIntro->show();
+	updateControlsGeometry();
+}
+
 void Suggestions::applySearchQuery() {
 	const auto key = _key.current();
 	const auto controller = _mediaLists[key].wrap->controller();
@@ -1926,7 +2164,10 @@ void Suggestions::switchTab(Key key) {
 }
 
 void Suggestions::ensureContent(Key key) {
-	if (key.tab != Tab::Downloads && key.tab != Tab::Media) {
+	if (key.tab == Tab::Posts) {
+		setPostsSearchQuery(QString());
+		return;
+	} else if (key.tab != Tab::Downloads && key.tab != Tab::Media) {
 		return;
 	}
 	auto &list = _mediaLists[key];
@@ -1958,6 +2199,7 @@ void Suggestions::startSlideAnimation(Key was, Key now) {
 			case Tab::Chats: return _chatsScroll.get();
 			case Tab::Channels: return _channelsScroll.get();
 			case Tab::Apps: return _appsScroll.get();
+			case Tab::Posts: return _postsScroll.get();
 			}
 			return _mediaLists[key].wrap;
 		};
@@ -2009,6 +2251,7 @@ void Suggestions::startShownAnimation(bool shown, Fn<void()> finish) {
 	_chatsScroll->hide();
 	_channelsScroll->hide();
 	_appsScroll->hide();
+	_postsScroll->hide();
 	for (const auto &[key, list] : _mediaLists) {
 		list.wrap->hide();
 	}
@@ -2028,6 +2271,7 @@ void Suggestions::finishShow() {
 	_chatsScroll->setVisible(key == Key{ Tab::Chats });
 	_channelsScroll->setVisible(key == Key{ Tab::Channels });
 	_appsScroll->setVisible(key == Key{ Tab::Apps });
+	_postsScroll->setVisible(key == Key{ Tab::Posts });
 	for (const auto &[mediaKey, list] : _mediaLists) {
 		list.wrap->setVisible(key == mediaKey);
 		if (key == mediaKey) {
@@ -2042,6 +2286,8 @@ void Suggestions::finishShow() {
 		reinstallSwipe(_channelsScroll.get());
 	} else if (key == Key{ Tab::Apps }) {
 		reinstallSwipe(_appsScroll.get());
+	} else if (key == Key{ Tab::Posts }) {
+		reinstallSwipe(_postsScroll.get());
 	}
 }
 
@@ -2055,6 +2301,7 @@ std::vector<Suggestions::Key> Suggestions::TabKeysFor(
 		{ Tab::Chats },
 		{ Tab::Channels },
 		{ Tab::Apps },
+		{ Tab::Posts },
 		{ Tab::Media, MediaType::Photo },
 		{ Tab::Media, MediaType::Video },
 		{ Tab::Downloads },
@@ -2119,6 +2366,16 @@ void Suggestions::updateControlsGeometry() {
 	_appsScroll->setGeometry(content);
 	_appsContent->resizeToWidth(w);
 
+	_postsScroll->setGeometry(content);
+	_postsWrap->resizeToWidth(w);
+	if (_postsSearchIntro) {
+		_postsSearchIntro->setGeometry(0, 0, w, height() - tabs);
+	} else if (_postsContent) {
+		_postsContent->resizeToWidth(w);
+		_postsContent->setMinimumHeight(height() - tabs);
+		_postsContent->refresh();
+	}
+
 	const auto expanding = false;
 	for (const auto &[key, list] : _mediaLists) {
 		const auto full = !list.wrap->scrollBottomSkip();
@@ -2138,7 +2395,8 @@ auto Suggestions::setupRecentPeers(RecentPeersList recentPeers)
 	const auto controller = lifetime().make_state<RecentsController>(
 		_controller,
 		std::move(recentPeers),
-		[=](not_null<PeerData*> p) { _openBotMainAppRequests.fire_copy(p); });
+		[=](not_null<PeerData*> p) { _openBotMainAppRequests.fire_copy(p); },
+		[=] { _closeRequests.fire({}); });
 
 	const auto addToScroll = [=] {
 		return _topPeersWrap->toggled() ? _topPeers->height() : 0;
@@ -2296,6 +2554,9 @@ auto Suggestions::setupRecommendations() -> std::unique_ptr<ObjectList> {
 auto Suggestions::setupRecentApps() -> std::unique_ptr<ObjectList> {
 	const auto controller = lifetime().make_state<RecentAppsController>(
 		_controller);
+	controller->setCloseCallback([=] {
+		_closeRequests.fire({});
+	});
 	_recentAppsShows = [=](not_null<PeerData*> peer) {
 		return controller->shown(peer);
 	};
