@@ -9,6 +9,10 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 
 #include "styles/style_widgets.h"
 
+namespace Calls {
+class GroupCall;
+} // namespace Calls
+
 namespace Data {
 class Session;
 } // namespace Data
@@ -33,10 +37,19 @@ public:
 		not_null<Data::Session*> owner,
 		rpl::producer<int> topOffset,
 		Fn<bool(not_null<const Element*> view)> mine);
+	PaidReactionToast(
+		not_null<Ui::RpWidget*> parent,
+		not_null<Data::Session*> owner,
+		rpl::producer<int> topOffset,
+		Fn<bool(not_null<Calls::GroupCall*> call)> mine);
 	~PaidReactionToast();
 
+	[[nodiscard]] rpl::producer<FullMsgId> shownForId() const;
+	[[nodiscard]] rpl::producer<Calls::GroupCall*> shownForCall() const;
+
 private:
 	bool maybeShowFor(not_null<HistoryItem*> item);
+	bool maybeShowFor(not_null<Calls::GroupCall*> call);
 	void showFor(
 		FullMsgId itemId,
 		int count,
@@ -44,6 +57,8 @@ private:
 		crl::time left,
 		crl::time total);
 
+	[[nodiscard]] FullMsgId idForCall(not_null<Calls::GroupCall*> call);
+
 	void setupLottiePreview(not_null<Ui::RpWidget*> widget, int size);
 	void clearHiddenHiding();
 
@@ -59,6 +74,10 @@ private:
 
 	std::vector<FullMsgId> _stack;
 
+	base::flat_map<FullMsgId, base::weak_ptr<Calls::GroupCall>> _idsForCalls;
+
+	rpl::variable<FullMsgId> _shownForId;
+
 	rpl::lifetime _lifetime;
 
 };
