@@ -84,6 +84,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "ui/widgets/labels.h"
 #include "ui/widgets/menu/menu_add_action_callback_factory.h"
 #include "ui/widgets/popup_menu.h"
+#include "ui/widgets/tooltip.h"
 #include "ui/wrap/fade_wrap.h"
 #include "window/themes/window_theme.h"
 #include "window/window_peer_menu.h"
@@ -103,6 +104,41 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 namespace Info::Profile {
 namespace {
 
+class Userpic final
+	: public Ui::AbstractButton
+	, public Ui::AbstractTooltipShower {
+public:
+	Userpic(QWidget *parent, Fn<bool()> hasStories)
+	: Ui::AbstractButton(parent)
+	, _hasStories(std::move(hasStories)) {
+		installEventFilter(this);
+	}
+
+	QString tooltipText() const override {
+		return _hasStories() ? tr::lng_view_button_story(tr::now) : QString();
+	}
+
+	QPoint tooltipPos() const override {
+		return QCursor::pos();
+	}
+
+	bool tooltipWindowActive() const override {
+		return Ui::AppInFocus() && Ui::InFocusChain(window());
+	}
+
+protected:
+	bool eventFilter(QObject *obj, QEvent *e) override {
+		if (obj == this && e->type() == QEvent::Enter && _hasStories()) {
+			Ui::Tooltip::Show(1000, this);
+		}
+		return Ui::AbstractButton::eventFilter(obj, e);
+	}
+
+private:
+	Fn<bool()> _hasStories;
+
+};
+
 constexpr auto kWaitBeforeGiftBadge = crl::time(1000);
 constexpr auto kGiftBadgeGlares = 3;
 constexpr auto kMinPatternRadius = 8;
@@ -462,7 +498,7 @@ void TopBar::adjustColors(const std::optional<QColor> &edgeColor) {
 			&& (kMinContrast > Ui::CountContrast(color->c, *edgeColor));
 	};
 	const auto shouldOverrideTitle = shouldOverride(_title->st().textFg);
-	const auto shouldOverrideStatus = shouldOverride(_status->st().textFg);
+	const auto shouldOverrideStatus = shouldOverrideTitle; // shouldOverride(_status->st().textFg);
 	_title->setTextColorOverride(shouldOverrideTitle
 		? std::optional<QColor>(st::groupCallMembersFg->c)
 		: std::nullopt);
@@ -908,32 +944,9 @@ void TopBar::setupActions(not_null<Window::SessionController*> controller) {
 
 void TopBar::setupUserpicButton(
 		not_null<Window::SessionController*> controller) {
-	_userpicButton = base::make_unique_q<Ui::AbstractButton>(this);
-
-	const auto invalidate = [=] {
-		_userpicUniqueKey = InMemoryKey();
-		_userpicButton->setAttribute(
-			Qt::WA_TransparentForMouseEvents,
-			!_peer->userpicPhotoId() && !_hasStories);
-		updateVideoUserpic();
-	};
-
-	rpl::single(
-		rpl::empty_value()
-	) | rpl::then(
-		_peer->session().changes().peerFlagsValue(
-			_peer,
-			Data::PeerUpdate::Flag::Photo
-				| Data::PeerUpdate::Flag::FullInfo) | rpl::to_empty
-	) | rpl::start_with_next(invalidate, lifetime());
-
-	if (const auto broadcast = _peer->monoforumBroadcast()) {
-		_peer->session().changes().peerFlagsValue(
-			broadcast,
-			Data::PeerUpdate::Flag::Photo
-				| Data::PeerUpdate::Flag::FullInfo
-		) | rpl::to_empty | rpl::start_with_next(invalidate, lifetime());
-	}
+	_userpicButton = base::make_unique_q<Userpic>(
+		this,
+		[=] { return _hasStories; });
 
 	const auto openPhoto = [=, peer = _peer] {
 		if (const auto id = peer->userpicPhotoId()) {
@@ -967,13 +980,19 @@ void TopBar::setupUserpicButton(
 		return true;
 	};
 
-	const auto isContact = [=, peer = _peer] {
+	const auto canChangePhoto = [=, peer = _peer] {
 		if (const auto user = peer->asUser()) {
 			return user->isContact()
 				&& !user->isSelf()
 				&& !user->isInaccessible()
 				&& !user->isServiceUser();
 		}
+		if (const auto chat = peer->asChat()) {
+			return chat->canEditInformation();
+		}
+		if (const auto channel = peer->asChannel()) {
+			return channel->canEditInformation();
+		}
 		return false;
 	};
 
@@ -987,9 +1006,60 @@ void TopBar::setupUserpicButton(
 		return false;
 	};
 
-	const auto choosePhotoCallback = [=](Ui::UserpicButton::ChosenType type) {
+	const auto hasMenu = [=] {
+		if (canChangePhoto()) {
+			return true;
+		}
+		if (canSuggestPhoto()) {
+			return true;
+		}
+		if (_hasStories || canReport()) {
+			return !!_peer->userpicPhotoId();
+		}
+		return false;
+	};
+
+	const auto invalidate = [=] {
+		_userpicUniqueKey = InMemoryKey();
+		const auto hasLeftButton = _peer->userpicPhotoId() || _hasStories;
+		_userpicButton->setAttribute(
+			Qt::WA_TransparentForMouseEvents,
+			!hasLeftButton && !hasMenu());
+		_userpicButton->setPointerCursor(hasLeftButton);
+		updateVideoUserpic();
+		_peer->session().downloaderTaskFinished(
+		) | rpl::filter([=] {
+			return !Ui::PeerUserpicLoading(_userpicView);
+		}) | rpl::start_with_next([=] {
+			update();
+			_userpicLoadingLifetime.destroy();
+		}, _userpicLoadingLifetime);
+		Ui::PostponeCall(this, [=] {
+			update();
+		});
+	};
+
+	rpl::single(
+		rpl::empty_value()
+	) | rpl::then(
+		_peer->session().changes().peerFlagsValue(
+			_peer,
+			Data::PeerUpdate::Flag::Photo
+				| Data::PeerUpdate::Flag::FullInfo) | rpl::to_empty
+	) | rpl::start_with_next(invalidate, lifetime());
+
+	if (const auto broadcast = _peer->monoforumBroadcast()) {
+		_peer->session().changes().peerFlagsValue(
+			broadcast,
+			Data::PeerUpdate::Flag::Photo
+				| Data::PeerUpdate::Flag::FullInfo
+		) | rpl::to_empty | rpl::start_with_next(invalidate, lifetime());
+	}
+
+	using ChosenType = Ui::UserpicButton::ChosenType;
+
+	const auto choosePhotoCallback = [=](ChosenType type) {
 		return [=](QImage &&image) {
-			using ChosenType = Ui::UserpicButton::ChosenType;
 			auto result = Api::PeerPhoto::UserPhoto{
 				std::move(image),
 				0,
@@ -1010,12 +1080,12 @@ void TopBar::setupUserpicButton(
 		};
 	};
 
-	const auto editorData = [=](Ui::UserpicButton::ChosenType type) {
+	const auto editorData = [=](ChosenType type) {
 		const auto user = _peer->asUser();
 		const auto name = (user && !user->firstName.isEmpty())
 			? user->firstName
 			: _peer->name();
-		const auto phrase = (type == Ui::UserpicButton::ChosenType::Suggest)
+		const auto phrase = (type == ChosenType::Suggest)
 			? &tr::lng_profile_suggest_sure
 			: &tr::lng_profile_set_personal_sure;
 		return Editor::EditorData{
@@ -1024,7 +1094,7 @@ void TopBar::setupUserpicButton(
 				lt_user,
 				Ui::Text::Bold(name),
 				Ui::Text::WithEntities),
-			.confirm = ((type == Ui::UserpicButton::ChosenType::Suggest)
+			.confirm = ((type == ChosenType::Suggest)
 				? tr::lng_profile_suggest_button(tr::now)
 				: tr::lng_profile_set_photo_button(tr::now)),
 			.cropType = Editor::EditorData::CropType::Ellipse,
@@ -1032,7 +1102,7 @@ void TopBar::setupUserpicButton(
 		};
 	};
 
-	const auto chooseFile = [=](Ui::UserpicButton::ChosenType type) {
+	const auto chooseFile = [=](ChosenType type) {
 		base::call_delayed(
 			st::defaultRippleAnimation.hideDuration,
 			crl::guard(this, [=] {
@@ -1046,7 +1116,7 @@ void TopBar::setupUserpicButton(
 
 	const auto addFromClipboard = [=](
 			Ui::PopupMenu *menu,
-			Ui::UserpicButton::ChosenType type,
+			ChosenType type,
 			tr::phrase<> text) {
 		if (const auto data = QGuiApplication::clipboard()->mimeData()) {
 			if (data->hasImage()) {
@@ -1068,9 +1138,7 @@ void TopBar::setupUserpicButton(
 
 	_userpicButton->clicks() | rpl::start_with_next([=](
 			Qt::MouseButton button) {
-		if (button == Qt::RightButton
-			&& (_hasStories || canReport() || isContact())
-			&& _peer->userpicPhotoId()) {
+		if (button == Qt::RightButton && hasMenu()) {
 			*menu = base::make_unique_q<Ui::PopupMenu>(
 				this,
 				st::popupMenuWithIcons);
@@ -1095,32 +1163,48 @@ void TopBar::setupUserpicButton(
 					&st::menuIconReport);
 			}
 
-			if (isContact()) {
+			if (canChangePhoto()) {
 				if (!(*menu)->empty()) {
 					(*menu)->addSeparator(&st::expandedMenuSeparator);
 				}
-				(*menu)->addAction(
-					tr::lng_profile_set_photo_for(tr::now),
-					[=] { chooseFile(Ui::UserpicButton::ChosenType::Set); },
-					&st::menuIconPhotoSet);
-				addFromClipboard(
-					menu->get(),
-					Ui::UserpicButton::ChosenType::Set,
-					tr::lng_profile_set_photo_for_from_clipboard);
-				if (canSuggestPhoto()) {
+				const auto isUser = _peer->asUser();
+				if (isUser) {
 					(*menu)->addAction(
-						tr::lng_profile_suggest_photo(tr::now),
-						[=] {
-							chooseFile(
-								Ui::UserpicButton::ChosenType::Suggest);
-						},
-						&st::menuIconPhotoSuggest);
+						tr::lng_profile_set_photo_for(tr::now),
+						[=] { chooseFile(ChosenType::Set); },
+						&st::menuIconPhotoSet);
 					addFromClipboard(
 						menu->get(),
-						Ui::UserpicButton::ChosenType::Suggest,
-						tr::lng_profile_suggest_photo_from_clipboard);
+						ChosenType::Set,
+						tr::lng_profile_set_photo_for_from_clipboard);
+					if (canSuggestPhoto()) {
+						(*menu)->addAction(
+							tr::lng_profile_suggest_photo(tr::now),
+							[=] {
+								chooseFile(
+									ChosenType::Suggest);
+							},
+							&st::menuIconPhotoSuggest);
+						addFromClipboard(
+								menu->get(),
+								ChosenType::Suggest,
+								tr::lng_profile_suggest_photo_from_clipboard);
+					}
+				} else {
+					const auto channel = _peer->asChannel();
+					const auto isChannel = channel && !channel->isMegagroup();
+					(*menu)->addAction(
+						isChannel
+							? tr::lng_profile_set_photo_for_channel(tr::now)
+							: tr::lng_profile_set_photo_for_group(tr::now),
+						[=] { chooseFile(ChosenType::Set); },
+						&st::menuIconPhotoSet);
+					addFromClipboard(
+						menu->get(),
+						ChosenType::Set,
+						tr::lng_profile_set_photo_for_from_clipboard);
 				}
-				if (controller) {
+				if (controller && isUser) {
 					const auto done = [=](UserpicBuilder::Result data) {
 						auto result = Api::PeerPhoto::UserPhoto{
 							base::take(data.image),
@@ -2343,7 +2427,8 @@ void TopBar::paintPinnedToTopGifts(
 
 void TopBar::setupStoryOutline(const QRect &geometry) {
 	const auto user = _peer->asUser();
-	if (!user) {
+	const auto channel = _peer->asChannel();
+	if (!user && !channel) {
 		return;
 	}
 
@@ -2352,18 +2437,18 @@ void TopBar::setupStoryOutline(const QRect &geometry) {
 		rpl::merge(
 			rpl::single(rpl::empty_value()),
 			style::PaletteChanged(),
-			user->session().changes().peerUpdates(
+			_peer->session().changes().peerUpdates(
 				Data::PeerUpdate::Flag::StoriesState
 					| Data::PeerUpdate::Flag::ColorProfile
 			) | rpl::filter([=](const Data::PeerUpdate &update) {
-				return update.peer == user;
+				return update.peer == _peer;
 			}) | rpl::to_empty)
 	) | rpl::start_with_next([=](
 			std::optional<QColor> edgeColor,
 			rpl::empty_value) {
 		const auto geometry = QRectF(userpicGeometry());
 		const auto colorProfile
-			= user->session().api().peerColors().colorProfileFor(user);
+			= _peer->session().api().peerColors().colorProfileFor(_peer);
 		const auto hasProfileColor = colorProfile
 			&& colorProfile->story.size() > 1;
 		if (hasProfileColor) {
@@ -2381,16 +2466,17 @@ void TopBar::setupStoryOutline(const QRect &geometry) {
 
 void TopBar::updateStoryOutline(std::optional<QColor> edgeColor) {
 	const auto user = _peer->asUser();
-	if (!user) {
+	const auto channel = _peer->asChannel();
+	if (!user && !channel) {
 		return;
 	}
 
 	const auto hasActiveStories = (_source == Source::Preview)
 		? true
-		: user->hasActiveStories();
+		: (user ? user->hasActiveStories() : channel->hasActiveStories());
 	const auto hasLiveStories = (_source == Source::Preview)
 		? false
-		: user->hasActiveVideoStream();
+		: (user ? user->hasActiveVideoStream() : false);
 
 	if (_hasStories != hasActiveStories
 		|| _hasLiveStories != hasLiveStories) {
@@ -2429,8 +2515,8 @@ void TopBar::updateStoryOutline(std::optional<QColor> edgeColor) {
 		return;
 	}
 
-	const auto &stories = user->owner().stories();
-	const auto source = stories.source(user->id);
+	const auto &stories = _peer->owner().stories();
+	const auto source = stories.source(_peer->id);
 	if (!source) {
 		return;
 	}
