@@ -32,6 +32,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "core/application.h"
 #include "core/core_settings.h"
 #include "history/view/media/history_view_media_common.h"
+#include "ui/basic_click_handlers.h"
 #include "ui/chat/chat_style.h"
 #include "ui/effects/premium_graphics.h"
 #include "ui/effects/premium_top_bar.h"
@@ -457,6 +458,10 @@ void SetupPrivacy(
 		tr::lng_settings_bio_privacy(),
 		Key::About,
 		[] { return std::make_unique<AboutPrivacyController>(); });
+	add(
+		tr::lng_settings_saved_music_privacy(),
+		Key::SavedMusic,
+		[] { return std::make_unique<SavedMusicPrivacyController>(); });
 	add(
 		tr::lng_settings_groups_invite(),
 		Key::Invites,
@@ -604,20 +609,8 @@ void SetupLoginEmail(
 		{ &st::menuIconRecoveryEmail });
 	CreateRightLabel(button, std::move(label), st, std::move(text));
 
-	button->addClickHandler([=, email = std::move(email)] {
-		controller->uiShow()->show(Box([=](not_null<Ui::GenericBox*> box) {
-			Ui::ConfirmBox(box, Ui::ConfirmBoxArgs{
-				.text = tr::lng_settings_cloud_login_email_box_about(),
-				.confirmed = [=](Fn<void()> close) {
-					showOther(CloudLoginEmailId());
-					close();
-				},
-				.confirmText = tr::lng_settings_cloud_login_email_box_ok(),
-			});
-			box->getDelegate()->setTitle(rpl::duplicate(
-				email
-			) | rpl::map(Ui::Text::WrapEmailPattern));
-		}));
+	button->addClickHandler([=] {
+		UrlClickHandler::Open(u"tg://settings/login_email"_q);
 	});
 
 	const auto reloadOnActivation = [=](Qt::ApplicationState state) {
